#include <hidef.h>				// common defines and macros 
#include "derivative.h"			// derivative-specific definitions 
#include "sci_hc12.h"			// sci routines
#include "keypad.h"				// keypad routines
#include "lcd.h"					// lcd routines


void CRG_8MHz(void);
void ATD_EN(void);
void READ_POT(void);

#define SW1 PTJ_PTJ6
#define SW2 PTJ_PTJ7
#define LED1 PTS_PTS2
#define LED2 PTS_PTS3
#define ON  1
#define OFF 0
#define shiftP 54
#define shiftK 44

byte temp;
byte curATD=0;            // current ATD value


// Uses Optrex DMC-2043 LCD 
char str1[] = "Functional Test     ";   // 20 char line length
char str2[] = "AXIOM MANUFACTURING ";
char str3[] = "Key:      POT:      ";
char str4[] = "CMLU-12C32          ";

void main(void) {
	byte newresult=0, oldresult=0, n;
	byte curATDL, curATDH;
  unsigned char cKey_row, cKey_col, cRow_cnt;
  unsigned char cKey_new, cKey_old=0;
  word memstart, memend, m, p;
	
  const unsigned char cKey_code[] = {'1','2','3','A','4','5','6','B',
                                   '7','8','9','C','*','0','#','D'};

	COPCTL = 0;							// disable COP
  EnableInterrupts;
	CRG_8MHz();							// 8MHz bus clk
	ATD_EN();					  		// en ATD
	ensci();								// en UART0 19.2K,8,N,1
	InitKeypad();						// init keypad
	LCDInit();							// init LCD
	
	PTS = 0x0;							// init LEDs OFF
	DDRS = 0x0C;						// LED(2;1) output
	PERJ = 0xC0;						// en SW(2:1) pullup
	PIEJ = 0xC0;						// en PORTJ[7:6] interrupts
	PIFJ &= 0xC0;						// clear Int flags
	DDRM |= 0x02;						// PTM[1] output
	
	LCDputs(str1);
	LCDputs(str2);
	LCDputs(str3);
	LCDputs(str4);
//	temp = 1;
/*	
// test ext ram
	PEAR = 0x0C;						//set expanded wide mode
	MODE = 0xE2;
	EBICTL = 0x01;					// en stretch
	MISC = 0X0F;			 			// low flash off, stretch=3

	memstart = 0x4000;			// start of ext memory
	memend = 0x8000;				// end of mem test
	
	for (m=0; m<10;m++){
		p = ~m;								// compliment addr
		m = p;								// write value to mem loc'n
	}
*/	

	for(;;){

    //echo_com();                       // echo terminal input

// convert POT postion and convert value to ASCII for display on LCD.
// update output only if POT changed.  Some loss of fidelity.
    ATDCTL5_Cx = 0x07;    // start conversion
    if(!ATDSTAT0_SCF);    // wait until conversion complete
    for(n=0;n<0xFF;n++);   // delay
    if (ATDDR7L<curATDL || ATDDR7L>curATDH) {    // if POT reading changes
      LCD_Home();           // cursor to home 
      LCD_Cursor_Right(shiftP); // move cursor to POT output location
      curATD = ATDDR7L;      // capture current ATD reading
      LCDouthex(ATDDR7L);   // send POT results to LCD
      curATDL = curATD - 1;   // lo value
      curATDH = curATD + 1;   // hi value
    }
    
// scan keypad and output key press to LCD if updated.  Will not 
// update LCD if no key or same key pressed.  

    PTP = 0x01;           // set to col 1
		for(cKey_row=0;cKey_row<4;cKey_row++){     // scan all row's
			if(PTP & 0xF0){                   // if any sw pressed this row
				cKey_col = (PTP & 0xF0) >> 4;   // capture col num in low nibble
				if (cKey_col==4) cKey_col = 3;  
				if (cKey_col==8) cKey_col = 4;    // adjust row num
				cKey_col = cKey_col - 1;          // col count "0" based
				if (cKey_row==0)
					cRow_cnt = cKey_row;          // adjust row count
				else
					cRow_cnt = (cKey_row) * 0x04;   // adjust row count
					cKey_new = (*(cKey_code+(cRow_cnt+cKey_col)));
				if(cKey_old!=cKey_new){
					LCD_Home();                 // move cursor
					LCD_Cursor_Right(shiftK);   // move cursor to output KEY value
					LCDputch(cKey_new);					// send new key press
					cKey_old = cKey_new;        // save key press
				}
			}
			PTP = PTP << 1;
		}
	}
}


/**********************************************************
* READ_POT
*
* Read POT and display value on LCD if different.  If 
* unchanged, do nothing.  Move LCD cursor to position if 
* new data is ready to send out
**********************************************************/
void READ_POT(void){
  
  ATDCTL5_Cx = 0x07;    // start conversion
    if(!ATDSTAT0_SCF);    // wait until conversion complete
                          
//  if (curATD!=ATDDR7L) {    // if POT reading changes
  LCD_Home();           // cursor to home 
  LCD_Cursor_Right(shiftP); // move cursor to POT output location
  curATD = ATDDR7L;      // capture current ATD reading
  LCDouthex(ATDDR7L);   // send POT results to LCD
//    }
}
  
/**********************************************************
*	CRG_8MHz
*
*  Configure CRG module for 8MHz bus clock
*	For 16MHz XTAL - 
*	PLLCLK - 16MHz, Core Clk - 16MHz, Bus Clk - 8MHz
**********************************************************/
void CRG_8MHz(void){

	SYNR = 0x00;
	REFDV = 0x01;
	
	PLLCTL_PLLON = 1;					// enable PLL
	PLLCTL_AUTO = 1;					// enable autolock
	while(!CRGFLG_LOCK);				// wait for lock
	CRGFLG = 0xFF;						// clear CRG flags
	CLKSEL_PLLSEL = 1;				// select PLL output
}

/**********************************************************
*	ATD_EN
*
*	Configure ATD to sample CH7.  Write Channel Select Code
*	bits C[C:A] to start conversion sequence
*  output - 8B / right justified / unsigned
**********************************************************/
void ATD_EN(void){

	ATDCTL2_ADPU = 1;					// ATD on
	ATDCTL3_S8C = 1;					// 8 conversions / sequence
	ATDCTL4_SRES8 = 1;				// 8 bit results
	ATDCTL5_DJM = 1;					// right justified result
	ATDCTL5_DSGN = 0;					// unsigned result
	ATDCTL4_PRS = 0;					// bus clk div 2, 4 clks per sample
	ATDSTAT0_SCF = 1;					// clear flag
}

#pragma CODE_SEG NON_BANKED

/**********************************************************
*	kbi_isr
*
*	interrupt handler for PBSWs connected to PORTJ
**********************************************************/
void interrupt 24 kbi_isr(void) {
	DisableInterrupts;
	switch(PIFJ){
		case 0x40: LED1=!LED1; break;
		case 0x80: LED2=!LED2; break;
		default:  LED1=OFF; LED2=OFF;
	}
	PIFJ &= 0xC0;						// clear PBSW flags
	EnableInterrupts;
}


/**********************************************************
*	sci_isr
*
*	interrupt handler for UART0
**********************************************************/
void interrupt 20 sci_isr(void) {
	DisableInterrupts
	putch(GetSByte());				// echo COM input
	EnableInterrupts
}


/**********************************************************
*	keypad_isr
*
*	interrupt handler for UART0
*********************************************************
void interrupt 56 keypad_isr(void) {

  byte n;
  word m;
  DisableInterrupts
  n = getkeypad();					// get keypress
  lcd_cmd(2);               // cursor home
  LCD_Cursor_Right(shiftK); // move cursor
  LCDputch(n);						  // send keypress to LCD
  for (m=0;m<0xFFFF;m++);   // delay to allow LCD output to complete 
  PIFP = 0xFF;						  // clear int flags
  PTP = 0x0F;							  // arm columns for next keypress
  EnableInterrupts
}
*/